//------------------------------------------------------------------------------
/*
 * Project name:
     Frequency Meter 50MHz
 * Copyright:
     Copyright  2009 Ar2r. All rights reserved.
 * Description:
     PIC counts input, converts to decimal value and show result to display.
 * Test configuration:
     MCU:             PIC16F877
     Dev.Board:       Custom
     Oscillator:      HS, 4.194304 MHz
     Ext. Modules:    LCD 2x16
     SW:              MikroC v8.2
 * NOTES:
     None.
*/
//------------------------------------------------------------------------------
#include "Ar2r.h"

unsigned long   cntr, freq=0;          // Number of RB0 transition ; frequency
unsigned char   str[10];               // Display result string
unsigned short  state;                 // If interrupt occured =1
//------------------------------------------------------------------------------
void Count(void)
{
    freq=(TMR1H<<8)+TMR1L;             // Calculates decimal value of frequency
    freq=(cntr*65536)+freq;
}
//------------------------------------------------------------------------------
void ResetClk(void)
{
    PORTD.F0 = 1;                      // Reset Clock
    PORTD.F0 = 0;                      // Enable Clock
}
//------------------------------------------------------------------------------
void interrupt(void)
{
    if(INTCON.INTF)                    // External interrupt occured
        {
            state=1;
            INTCON.INTF = 0;           // Clear interrupt flag to enable next call
        }
    if(PIR1.TMR1IF)                    // TMR1 overfow occured
        {
            cntr++ ;                   // inc. transition counter
            PIR1.TMR1IF = 0 ;          // Clear interrupt flag to enable next call on overflow
        }
}
//------------------------------------------------------------------------------
void main()
{
    Lcd_Init(&PORTB);                  // Initialize display on PORTB
    TRISB.F0   = 1;                    // Set PORTB B0 is input (IRQ to stop count and display freq.)
    TRISC.F0   = 1;                    // Set PORTC C0 is input (count input freq.)
    TRISD.F0   = 0;                    // Set PORTD D0 is output (reset pin)
    OPTION_REG = 0b11000000;
    T1CON      = 0b00000111;
    
    Lcd_Cmd(LCD_CURSOR_OFF);           // Turn cursor off
    Ar2rSplash();                      // You are GAY if you remove this line :D
    LCD_Cmd(LCD_CLEAR);                // Send command  to LCD (clear LCD)
    
    PIE1   = 0b00000001;               // TMR1IE enabled
    INTCON = 0b11010000;               // INTE, PEIE and GIE enabled
    ResetClk();                        // Reset external timebase

    while (1)                                    // Endless loop
        {
            if(state==1)                         // If external interrupt occured RB0
                {
                    INTCON.GIE = 0;              // Stop all interrupts
                    PIE1       = 0;

                    Count();                     // Calculates decimal value of frequency
                    Lcd_Out(1,4, "Frequency");   // Print text to LCD
                    LongToStr(freq, str);
                    Lcd_Out(2,1, str);           // Print text to LCD
                    Lcd_Chr_CP(' ');
                    Lcd_Chr_CP('H');
                    Lcd_Chr_CP('z');

                    cntr = 0;                    // Clear counters
                    TMR1H= 0;
                    TMR1L= 0;
                    state=0;

                    PIE1   = 0b00000001;         // TMR1IE enabled
                    INTCON = 0b11010000;         // INTE, PEIE and GIE enabled

                    ResetClk();                  // Reset external timebase
                }
        }
}
//------------------------------------------------------------------------------
